<?php
namespace Controllers; 

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;

class Ger80300DemonstrativoResultadoOperacionalController extends \Zeedhi\Framework\Controller\Report{
    
    protected $entityManager;
    protected $birtStrategy;
    protected $newCode;
    protected $environment;
    protected $connection;

    public function __construct($reportService, $entityManager, $birtStrategy, $newCode, $environment, $connection) {

        $this->entityManager       = $entityManager;
        $this->birtStrategy        = $birtStrategy;
        $this->reportService       = $reportService;
        $this->newCode             = $newCode;
        $this->environment         = $environment;
        $this->connection          = $connection;
    }
    
    protected $reportMapping = array (
        'GER80300' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        
                        "FILIAL"  => "P_CDFILIAL",
                        "DATAINI"   => "P_DATAINI",
                        "DATAFIM"   => "P_DATAFIN",
                        "NRORG"     => "P_NRORG"
                    )
            ),
        'GER80301' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        
                        "FILIAL"  => "P_CDFILIAL",
                        "DATAINI"   => "P_DATAINI",
                        "DATAFIM"   => "P_DATAFIN",
                        "NRORG"     => "P_NRORG"
                    )
            ),
        'GER80302' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        "CDCENTRAL"  => "P_CDCENTRAL",
                        "DATAINI"    => "P_DATAINI",
                        "DATAFIM"    => "P_DATAFIN",
                        "NRLOPER"    => "P_OPER",
                        "NRLISTA"    => "P_LISTAGREG",
                        "PERCENTFAT" => "P_PERCENT",
                        "NRORG"      => "P_NRORG",
                        "DETDIARIO"  => "P_DETDIARIO",
                        "DETUNIDADE" => "P_DETUNIDADE"
                    )
            ),
        'GER80303' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        "CONSOLIDADO"  => "P_CONSOL",      
                        "DATAINI"      => "P_DATAINI",
                        "DATAFIM"      => "P_DATAFIN",
                        "NRLOPER"      => "P_OPER",
                        "NRLISTA"      => "P_NRLFIL",
                        "MODELO"       => "P_MODELO",
                        "PERCENTFAT"   => "P_PERCENT",
                        "EXIBE"        => "P_EXIBRELACUNI",
                        "NRORG"        => "P_NRORG"
                    )
            ),
        'GER80304' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        "CONSOLIDADO"  => "P_CONSOL",      
                        "DATAINI"      => "P_DATAINI",
                        "DATAFIM"      => "P_DATAFIN",
                        "NRLOPER"      => "P_OPER",
                        "NRLISTA"      => "P_NRLFIL",
                        "MODELO"       => "P_MODELO",
                        "PERCENTFAT"   => "P_PERCENT",
                        "EXIBE"        => "P_EXIBRELACUNI",
                        "NRORG"        => "P_NRORG"
                    )
            ),
        'GER01200' => 
            array (
                'strategy' => 'BIRT',
                'parameterMapping' => 
                    array (
                        "FILIAL"    => "P_CDFILIAL",
                        "DATAINI"   => "P_DATAINI",
                        "DATAFIM"   => "P_DATAFIN",
                        "NRORG"     => "P_NRORG"
                    )
            )
    );
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $row['__report_name'] = $row['REPORT_NAME'];
        if($row['__report_name'] != 'GER80303' && $row['__report_name'] != 'GER80304' && $row['__report_name'] != 'GER01200' && $row['__report_name'] != 'GER80302'){
            $row['FILIAL'] = isset($row['CDFILIAL']) ? $row['CDFILIAL'][0] : null;
        }
        
        if (isset($row["CDFILLIST"]) && $row["CDFILLIST"] != "") {
            $row['NRLISTA'] = $this->criarListaValores(implode('_', $row["CDFILLIST"]));
        } else if(isset($row["CDFILIALADM"]) && $row["CDFILIALADM"] != "") {
            $row['FILIAL'] = $this->criarListaValores(implode('_', $row["CDFILIALADM"]));
        } else { 
            $result = $this->getUnidades($row, $row['__report_name'] === 'GER80302');
            if ($result) 
                $row['NRLISTA'] = $this->criarListaValores(implode('_', $result));
            else
                throw new \Exception("Não há unidades disponíveis para o filtro informado");
        }
        $row['NRLOPER'] = $this->criarListaValores($this->environment->getCdOperador());
        $row['NRORG'] = $this->environment->getNrorg();
        
        $row['EXIBE'] = $row["EXIBRELACUNI"].$row["EXIBUNINAORELAC"];
        if (isset($row["CDFILLIST"]))
            $row['EXIBE'] = $row['EXIBE'].'S';
            
        $this->birtStrategy->setReportFormat($row["FORMAT"]);
        $this->birtStrategy->setReportLocale($row["LOCALE"]);
        $this->birtStrategy->setReportTimezone($row["TIMEZONE"]);
    }
    
    public function validaData($row) {
    	$params = array(
    		"P_DTINICIAL" => $row["DATAINI"],
    		"P_DTFINAL" => $row["DATAFIM"],
    		"P_CDFILIAL" => $row["CDFILIAL"],
    		"P_DTINICIAL" => $row["DATAINI"],
    		"P_DTFINAL" => $row["DATAFIM"]
    	);
    	
    	$sql = "SELECT DTFILIALROP, DTINICROP, DTFINALROP,  DTGERAROP,
                       NRROPFILIAL,
                       SUBSTR(:P_DTINICIAL,1,15) AS DATAINICIAL,
                       SUBSTR(:P_DTFINAL,1,15) AS DATAFINAL
                FROM ROPFILIAL R
                WHERE
                   R.CDFILIAL  = :P_CDFILIAL         AND
                   DTFILIALROP BETWEEN :P_DTINICIAL AND :P_DTFINAL";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);  
    	
    	return $query->fetchAll()["CDFILIAL"];
    	
    }
    
    public function getUnidades($row, $agregadas) {
        if (!$agregadas) {
            $params = array(
                "P_CDOPERADOR" => $this->environment->getCdOperador(),
                "P_CDEMPRESA" => $row["CDEMPRESA"],
                "P_CDAREAGERE" => $row["CDAREAGERE"],
                "P_CDSEGMER" => $row["CDSEGMER"],
                "P_CDSUPERVISOR" => $row["CDSUPERVISOR"],
                "P_CDGRUPFILI" => $row["CDGRUPFILI"]
            );
            $sql = "SELECT FO.CDFILIAL
                      FROM FILIOPER FO, FILIAL F, FILGRUPFILI G
                     WHERE FO.CDOPERADOR = :P_CDOPERADOR
                       AND FO.CDFILIAL = F.CDFILIAL
                       AND FO.CDFILIAL = G.CDFILIAL(+)
                       AND ((F.CDEMPRESA = :P_CDEMPRESA) OR ('T' = NVL(:P_CDEMPRESA, 'T')))
                       AND ((F.CDAREAGERE = :P_CDAREAGERE) OR ('T' = NVL(:P_CDAREAGERE, 'T')))
                       AND ((F.CDSEGMER = :P_CDSEGMER) OR ('T' = NVL(:P_CDSEGMER, 'T')))
                       AND ((F.CDSUPERVISOR = :P_CDSUPERVISOR) OR ('T' = NVL(:P_CDSUPERVISOR, 'T')))
                       AND ((G.CDGRUPFILI = :P_CDGRUPFILI) OR ('T' = NVL(:P_CDGRUPFILI, 'T')))
                     GROUP BY FO.CDFILIAL
                     ORDER BY FO.CDFILIAL";
        } else {
            $params = array(
                "P_CDOPERADOR" => $this->environment->getCdOperador(),
                "P_CDCENTRAL" => $row["CDCENTRAL"]
            );
            $sql = "SELECT CDFILIAL
                      FROM FILIAL
                     WHERE CDFILIAL = :P_CDCENTRAL
                        OR CDFILIAL IN (SELECT FO.CDFILIAL
                                          FROM FILIOPER FO, FILIAL F
                                         WHERE FO.CDOPERADOR = :P_CDOPERADOR
                                           AND FO.CDFILIAL = F.CDFILIAL
                                           AND F.CDFILCENTPME = :P_CDCENTRAL
                                         GROUP BY FO.CDFILIAL)
                     ORDER BY CDFILIAL";
        }
                 
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    	
    	$arr = array();
    	$retorno = $query->fetchAll();
    	
    	foreach($retorno as $linha){
    	    array_push($arr, $linha["CDFILIAL"]);
    	}
    	
    	return $arr;
    }
    
    //cria lista de valores recebendo string como parametro
    public function criarListaValores($valores){
        $this->entityManager->beginTransaction();
        $lista = explode('_',$valores);
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
        try{
            foreach($lista as $key){
                $this->insertLista($nrlista, $key);
            }
            $this->entityManager->commit();
            return $nrlista;
        } catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    //insere lista no banco
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function verificaApuracao (DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        if(isset($row['CDFILIAL']) && isset($row['PERIODO'])){
            
            $query = "SELECT COUNT (*) AS QTD
                            FROM DATA D
                        WHERE D.DTDATA BETWEEN :P_DTINICIAL AND :P_DTFINAL
                            AND NOT EXISTS (SELECT 1
                                            FROM FILIALROP F
                                            WHERE F.CDFILIAL = :P_CDFILIAL
                                                AND F.DTFILIALROP BETWEEN :P_DTINICIAL AND :P_DTFINAL
                                                AND F.DTFILIALROP = D.DTDATA)";
            $params = array(
                "P_CDFILIAL"  => $row['CDFILIAL'][0],
                "P_DTINICIAL" =>$row['PERIODO']['start'],
                "P_DTFINAL" =>$row['PERIODO']['end']
            );
            $stmt = $this->connection->prepare($query);
            $stmt->execute($params);
            $data =  $stmt->fetchAll();
            $response->addDataSet(new DataSet("datas_nao_apuradas", $data));

        }
    }

    public function getDatasApuradas (DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('CDFILIAL', $row)){
            
            $query = "SELECT DTAPURADA
                                FROM (
                                    SELECT DISTINCT 
                                        TO_CHAR(DTFILIALROP, 'DD/MM/YYYY') AS DTAPURADA  , DTFILIALROP
                                            FROM FILIALROP 
                                            WHERE CDFILIAL = :P_CDFILIAL
                                )
                            ORDER BY DTFILIALROP";
            $params = array(
                "P_CDFILIAL"  => $row['CDFILIAL'][0]
            );
            $stmt = $this->connection->prepare($query);
            $stmt->execute($params);
            $data =  $stmt->fetchAll();
            $data = $this->array_change_key_case_recursive($data);
            $response->addDataSet(new DataSet("datas_apuradas", $data));

        }
    }

    function array_change_key_case_recursive($arr) {
        return array_map(function($item){
            if(is_array($item))
                $item = $this->array_change_key_case_recursive($item);
            return $item;
        },array_change_key_case($arr));
    }
    
    /**
     * Creates a temporary folder inside the ReportsTemp folder, generates all BIRT files 
     * referring to the units chosen by the user saving them in this folder, a .zip file is 
     * generated from there and then returns the file to the front allowing the download, as 
     * soon as Temporary folders created are deleted along with their contents.
     *
     * @param Request $request – Object that receives data from the caller.
     * @param Response $response –  Object that sends back data to the caller.
     * 
     * @return file
     * @throws Exception
     *
     * @access public
     * @author Grazielle Ferreira Antunes
     */
    
    public function downloadFolderReports (DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $time = time();
            
            $arrayPathReports = [];
            $row['NRORG'] = $this->environment->getNrorg();
            $filiais = explode(",", $row['CDFILIAL']);
            $dataini  = str_replace("/","",$row['DATAINI']);
            $datafim  = str_replace("/","",$row['DATAFIM']);
            
            $pathZip = '/tmp/' . 'DRO_' . $dataini . '_' . $datafim . '_' . $time . '.zip';
            
            $this->birtStrategy->setReportFormat($row["FORMAT"]);
            $this->birtStrategy->setReportLocale($row["LOCALE"]);
            $this->birtStrategy->setReportTimezone($row["TIMEZONE"]); 
            
            foreach($filiais as $filial) {
                $row['FILIAL'] = $filial;
                $reportURL = parent::processReport($request);
        		$pathReport = "/tmp/" . $filial . '_' . $dataini . '_' . $datafim . '_' . $time . '.' . $row["FORMAT"];
        	    
        	    file_put_contents($pathReport, fopen($reportURL, 'r'));
        		array_push($arrayPathReports, $pathReport);
            }

            $zip = new \ZipArchive;
            if( $zip->open( $pathZip , \ZipArchive::CREATE )  === true){
                $options = array('remove_all_path' => TRUE);
        	    
                foreach($arrayPathReports as $report) {
                    $zip->addFile($report, basename($report));
                }
                  
                $zip->close();
    
                $file = new DTO\Response\File($pathZip, true, null, 'application/zip');
                $response->setFileToDownload($file);
            }
                
            foreach($arrayPathReports as $report) {
                unlink($report);
            }
            
        } catch (\Exception $e) {
            if(isset($arrayPathReports) && count($arrayPathReports)) {
                foreach($arrayPathReports as $report) {
                    unlink($report);
                }
            }
            throw $e;
        }
    }
}